#!/usr/bin/env python3

import os
from io import BytesIO
from os.path import dirname, isdir, join

import pytest  # noqa

from miditk.common.constants import END_OF_TRACK, META_EVENT, POLY_PRESSURE
from miditk.smf.writer import MidiFileWriter


def test_write_type0():
    """Write MIDI file with the following events:

    format: 0, # of tracks: 1, division: 480
    ----------------------------------

    Start of track #0
    Sequence name - 'Type 0'
    Tempo: val:500000 (125.0 bpm)
    Time signature: 4/4 24 8
    Note on - ch:00, note:48h, vel:64h time:0
    Note off - ch:00, note:48h, vel:40h time:480
    End of track
    End of file

    """
    cmpfn = join(dirname(__file__), 'testdata', 'midiout.mid')
    outdir = join(dirname(__file__), 'testoutput')
    outfn = join(outdir, 'midiout.mid')

    if not isdir(outdir):
        os.makedirs(outdir)

    with open(outfn, 'wb') as smf:
        midi = MidiFileWriter(smf)

        midi.header(0, 1, 480)

        midi.start_of_track()
        midi.sequence_name('Type 0')
        midi.tempo(750000)
        midi.time_signature(4, 2, 24, 8)
        ch = 0

        for i in range(127):
            midi.note_on(ch, i, 0x64)
            midi.update_ticks(96)
            midi.note_off(ch, i, 0x40)
            midi.update_ticks(0)

        midi.update_ticks(0)
        midi.end_of_track()
        midi.eof()

    with open(outfn, 'rb') as out, open(cmpfn, 'rb') as cmp:
        assert cmp.read() == out.read()


def test_write_sysex_issue_7():
    smf = BytesIO()
    midi = MidiFileWriter(smf)
    midi.header(format=0, num_tracks=1, tick_division=96)
    midi.start_of_track(track=0)
    midi.system_exclusive(b'\xF0\xFD\x01\x02\x03\xF7')
    midi.end_of_track()
    midi.eof()

    assert b'\xf0\x07\xf0\xfd\x01\x02\x03\xf7' in smf.getvalue()


def test_write_poly_pressure():
    smf = BytesIO()
    channel = 10
    midi = MidiFileWriter(smf)
    midi.header(format=0, num_tracks=1, tick_division=96)
    midi.start_of_track(track=0)
    midi.poly_pressure(channel, 60, 127)
    midi.end_of_track()
    midi.eof()

    assert smf.getvalue().endswith(bytes([
        0, POLY_PRESSURE | channel, 60, 127,    # Poly pressure event
        0, META_EVENT, END_OF_TRACK, 0          # End of track meta event (len 0)
    ]))



#!/usr/bin/env python3
"""Example script which transposes all note on/off event by one octave."""

from os.path import abspath, dirname, join
from miditk.smf import MidiFileReader, MidiFileWriter


# MidiFileWriter is a sub-class of NullMidiEventHandler
class Transposer(MidiFileWriter):
    """Transpose note values of all note on/off events by 1 octave."""

    def _transp(self, ch, note):
        # don't transpose notes on the drum channel
        if ch != 9:
            note = min(127, note + 12)

        return note

    def note_on(self, channel, note, velocity):
        note = self._transp(channel, note)
        super().note_on(channel, note, velocity)

    def note_off(self, channel, note, velocity):
        note = self._transp(channel, note)
        super().note_off(channel, note, velocity)


if __name__ == '__main__':
    parentdir = dirname(dirname(abspath(__file__)))
    # This is a small type 0 midi file, with only two midi events on track #0.
    infilename = join(parentdir, 'tests', 'testdata', 'minimal.mid')
    # The output is written to the tests/testoutput directory
    outfilename = join(parentdir, 'tests', 'testoutput', 'transposed.mid')

    # Create parser and event handler
    with open(outfilename, 'wb') as smf:
        midiout = Transposer(smf)
        midiin = MidiFileReader(infilename, midiout)
        # now do the processing
        midiin.read()



import logging

from .api import BaseMidiEventHandler, MidiEvent, NullMidiEventHandler  # noqa: F401
from .converters import tobytestr  # noqa: F401
from .converters import (  # noqa: F401
    read_bew,
    read_varlen,
    sizeof_varlen,
    tointseq,
    write_bew,
    write_varlen,
)
from .parser import MidiFileParser  # noqa: F401
from .reader import MidiFileReader  # noqa: F401
from .sequence import MidiSequence  # noqa: F401
from .version import version as __version__  # noqa: F401
from .writer import BaseMidiFileWriter, MidiFileWriter  # noqa: F401


class NullHandler(logging.Handler):
    def emit(self, record):
        pass


logging.getLogger(__name__).addHandler(NullHandler())



#!/usr/bin/env python3
"""Parse a standard MIDI file into a MidiSequence instance

MidiSequence is a container for a sequence of timed MidiEvent instances,
which offers several convenience methods to access information about the
MIDI file properties and events.

"""

import sys

from miditk.smf import MidiSequence


# Do parsing
sequence = MidiSequence.fromfile(sys.argv[1])

# Print some info from the MIDI file header,
# e.g. number of tracks, events sequence name.
print("Sequence: ", sequence)


# Print a list of events with event type, data and timestamp
print("Sequence dump:")
sequence.dump_events()


# Iterate over all sysex events in track 0.
# If track is not specified, sysex_events() yields all sysex events
# in all tracks.
print("Sysex msgs in all tracks:")
for ev in sequence.sysex_events(track=0):
    print("Sysex event ({} bytes) @ {:.2}".format(len(ev.data), ev.timestamp))


print("Print events sorted by timestamp:")
# Iterate over all events sorted by timestamp and then track
for time, group in sequence.events_by_time():
    print(f"\n@ {time:.2f}:")
    for ev in group:
        print("- {:02X} {}".format(ev.type, " ".join(f"{b:02X}" for b in ev.data)))
